/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.actions;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.support.UISupport;

/**
 * Action for showing the desktop panel for the specified ModelItem
 * 
 * @author Ole.Matzura
 */

public class ShowDesktopPanelAction extends AbstractAction
{
	private final ModelItem target;

	public ShowDesktopPanelAction( String name, String description, ModelItem target )
   {
      super( name );
		this.target = target;
      
      putValue( Action.SHORT_DESCRIPTION, description );
      putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "ENTER"));
   }
	
   public void actionPerformed(ActionEvent arg0)
	{
   	if( target instanceof WsdlInterface )
   	{
   		try
			{
				((WsdlInterface)target).getWsdlContext().loadIfNecessary();
			}
			catch (Exception e)
			{
				e.printStackTrace();
				UISupport.showErrorMessage( e );
				return;
			}
   	}
   	
      UISupport.selectAndShow( target );
   }
}