/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl;

import java.util.ArrayList;
import java.util.List;

import com.eviware.soapui.model.iface.Attachment;
import com.eviware.soapui.model.iface.MessagePart;

/**
 * Descriptor for attachments
 * 
 * @author Ole.Matzura
 */

public final class WsdlAttachmentPart extends MessagePart.AttachmentPart
{
	public static final String ANONYMOUS_NAME = "<anonymous>";
	private String name;
	private List<String> contentTypes = new ArrayList<String>();
	private Attachment.AttachmentType type;
	private boolean anonymous;
	
	public WsdlAttachmentPart()
	{
		anonymous = true;
		name = ANONYMOUS_NAME;
	}
	
	public WsdlAttachmentPart(String name, List<String> types )
	{
		super();
		this.name = name;
		
		if( types != null )
			contentTypes.addAll( types );
	}
	
	public WsdlAttachmentPart( String name, String type )
	{
		this.name = name;
		if( type != null )
			contentTypes.add( type );
	}
	
	public String[] getContentTypes()
	{
		return contentTypes.toArray( new String[contentTypes.size()] );
	}

	public String getName()
	{
		return name;
	}
	
	public void addContentType( String contentType )
	{
		contentTypes.add( contentType );
	}
	
	public Attachment.AttachmentType getAttachmentType()
	{
		return type;
	}
	
	public void setType(Attachment.AttachmentType type)
	{
		this.type = type;
	}

	public String getDescription()
	{
		return name + " attachment; [" + getContentTypes() + "]";
	}

	public boolean isAnonymous()
	{
		return anonymous;
	}; 
}