/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.loadtest.data.actions;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.impl.wsdl.loadtest.log.LoadTestLog;
import com.eviware.soapui.impl.wsdl.loadtest.log.LoadTestLogEntry;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.support.UISupport;

/**
 * Simple loadtest log exporter, creates a comma-separated file containing a header row
 * and values for each log entry
 *  
 * @author Ole.Matzura
 */

public class ExportLoadTestLogAction extends AbstractAction
{
	private final LoadTestLog loadTestLog;

	public ExportLoadTestLogAction(LoadTestLog loadTestLog)
   {
		this.loadTestLog = loadTestLog;
		putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/export.gif"));
      putValue( Action.SHORT_DESCRIPTION, "Export current loadtest log to a file" );
   }
   
	public void actionPerformed(ActionEvent e)
	{
		try
		{
			if( loadTestLog.getSize() == 0 )
			{
				UISupport.showErrorMessage( "No data to export!" );
				return;
			}
			
         File file = UISupport.getFileDialogs().saveAs(this, "Select file for log export");
         if( file == null )
            return;

			int cnt = exportToFile(file);
			
			UISupport.showInfoMessage( "Saved " + cnt + " log entries to file [" + file.getName() + "]" );
		}
		catch (IOException e1)
		{
			e1.printStackTrace();
		}
   }

	public int exportToFile(File file) throws IOException
	{
		PrintWriter writer = new PrintWriter(file);
		writeHeader(writer);
		int cnt = writeLog(writer);
		writer.flush();
		writer.close();
		return cnt;
	}

	private int writeLog( PrintWriter writer)
	{
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		
		int c = 0;
		for( ; c < loadTestLog.getSize(); c++ )
		{
			LoadTestLogEntry logEntry = (LoadTestLogEntry) loadTestLog.getElementAt( c );
			writer.write( sdf.format( new Date(logEntry.getTimeStamp()) ));
			writer.write( ',' );
			writer.write( logEntry.getType() );
			writer.write( ',' );
			
			TestStep targetStep = logEntry.getTargetStep();
			if( targetStep != null )
				writer.write( targetStep.getName() );
			
			writer.write( ",\"" );
			writer.write( logEntry.getMessage() );
			writer.write( '"' );
			writer.println();
		}
		
		return c;
	}

	private void writeHeader(PrintWriter writer)
	{
		writer.println( "time,type,step,message");
	}
}