/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.mockoperation;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockRequest;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockResponse;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockResult;
import com.eviware.soapui.impl.wsdl.panels.attachments.MockAttachmentPanel;
import com.eviware.soapui.impl.wsdl.panels.mockoperation.actions.CreateEmptyMockResponseAction;
import com.eviware.soapui.impl.wsdl.panels.mockoperation.actions.CreateFaultMockResponseAction;
import com.eviware.soapui.impl.wsdl.panels.mockoperation.actions.OpenRequestForMockOperationAction;
import com.eviware.soapui.impl.wsdl.panels.mockoperation.actions.RecreateMockResponseAction;
import com.eviware.soapui.impl.wsdl.panels.mockoperation.actions.WSIValidateResponseAction;
import com.eviware.soapui.impl.wsdl.panels.request.StringToStringMapTableModel;
import com.eviware.soapui.impl.wsdl.panels.request.components.RequestMessageXmlEditor;
import com.eviware.soapui.impl.wsdl.panels.request.components.ResponseMessageXmlEditor;
import com.eviware.soapui.impl.wsdl.panels.request.components.SoapMessageXmlEditor;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.XmlDocument;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.views.source.XmlSourceEditorView;
import com.eviware.soapui.impl.wsdl.panels.teststeps.support.GroovyEditor;
import com.eviware.soapui.impl.wsdl.panels.teststeps.support.GroovyEditorModel;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.mock.MockRunner;
import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.settings.UISettings;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.actions.ChangeSplitPaneOrientationAction;
import com.eviware.soapui.support.components.JEditorStatusBarWithProgress;
import com.eviware.soapui.support.components.JXToolBar;
import com.eviware.soapui.support.swing.SoapUISplitPaneUI;
import com.eviware.soapui.support.types.StringToStringMap;
import com.eviware.soapui.support.xml.JXEditTextArea;
import com.eviware.soapui.ui.support.ModelItemDesktopPanel;
import com.jgoodies.forms.builder.ButtonBarBuilder;

/**
 * DesktopPanel for WsdlRequests
 * 
 * @author Ole.Matzura
 */

public class AbstractWsdlMockResponseDesktopPanel<T extends ModelItem, T2 extends WsdlMockResponse> extends ModelItemDesktopPanel<T>
{
	//private final static Log log = LogFactory.getLog(WsdlMockOperationDesktopPanel.class);
	private JEditorStatusBarWithProgress statusBar;
	private JButton splitButton;
	private MockRunner mockRunner;
	private JButton recreateButton;
	private JButton createEmptyButton;
	private JSplitPane requestSplitPane;
	private MoveFocusAction moveFocusAction;
	private ClosePanelAction closePanelAction = new ClosePanelAction();
	private RunScriptAction runScriptAction = new RunScriptAction();

	private SoapMessageXmlEditor requestEditor;
	private SoapMessageXmlEditor responseEditor;

	public WSIValidateResponseAction wsiValidateAction;

	private JTabbedPane requestTabs;
	private JPanel requestTabPanel;
	private JToggleButton tabsButton;

	public boolean responseHasFocus;

	private JTabbedPane requestEditorTabs;
	private JTabbedPane responseEditorTabs;

	private JPanel requestEditorTabPanel;
	private JPanel responseEditorTabPanel;
	private InternalPropertyChangeListener propertyChangeListener = new InternalPropertyChangeListener();
	private StringToStringMapTableModel responseHeadersModel;
	private JTable responseHeadersTable;
	private StringToStringMapTableModel requestHeadersModel;
	private GroovyEditor responseScriptEditor;
	private JButton createFaultButton;
	private T2 mockResponse;
	private JButton openRequestButton;;

	public AbstractWsdlMockResponseDesktopPanel(T modelItem)
	{
		super(modelItem);
	}
	
	protected void init( T2 mockResponse )
	{
		this.mockResponse = mockResponse;
		
		add(buildContent(), BorderLayout.CENTER);
		add(buildToolbar(), BorderLayout.NORTH);
		add(buildStatusLabel(), BorderLayout.SOUTH);

		setPreferredSize(new Dimension(600, 500));
		
		mockResponse.addPropertyChangeListener( propertyChangeListener );
	}

	protected WsdlMockResponse getMockResponse()
	{
		return mockResponse;
	}

	public final SoapMessageXmlEditor getRequestEditor()
	{
		return requestEditor;
	}

	public final SoapMessageXmlEditor getResponseEditor()
	{
		return responseEditor;
	}
	
	public MockRunner getSubmit()
	{
		return mockRunner;
	}

	protected JComponent buildStatusLabel()
	{
		statusBar = new JEditorStatusBarWithProgress();
		statusBar.setBorder( BorderFactory.createEmptyBorder( 1, 0, 0, 0 ));
		
		return statusBar;
	}

	public JEditorStatusBarWithProgress getStatusBar()
	{
		return statusBar;
	}

	protected JComponent buildContent()
	{
		requestSplitPane = UISupport.createHorizontalSplit();
		requestSplitPane.setResizeWeight(0.5);
		requestSplitPane.setBorder(null);
		
		splitButton = createActionButton(new ChangeSplitPaneOrientationAction(requestSplitPane), true);
		
		tabsButton = new JToggleButton( new ChangeToTabsAction() );
		tabsButton.setPreferredSize( UISupport.TOOLBAR_BUTTON_DIMENSION );
		
		openRequestButton = createActionButton(new OpenRequestForMockOperationAction( mockResponse.getMockOperation() ), true);
		recreateButton = createActionButton(new RecreateMockResponseAction( mockResponse ), true);
		createEmptyButton = createActionButton(new CreateEmptyMockResponseAction(mockResponse), true);
		createFaultButton = createActionButton(new CreateFaultMockResponseAction(mockResponse), true);

		moveFocusAction = new MoveFocusAction();
		wsiValidateAction = new WSIValidateResponseAction( mockResponse );

		requestEditor = buildRequestEditor();
		responseEditor = buildResponseEditor();
		JComponent scriptEditor = buildResponseScriptEditor();
		
		WsdlMockResult mockResult = mockResponse.getMockResult();
		int requestHeadersSize = mockResult == null ? -1 : mockResult.getMockRequest().getRequestHeaders().size();
		int requestAttachmentsSize = mockResult == null ? -1 : mockResult.getMockRequest().getRequestAttachments().length;
		
		requestEditorTabs = new JTabbedPane( JTabbedPane.BOTTOM );
		requestEditorTabs.setTabLayoutPolicy( JTabbedPane.SCROLL_TAB_LAYOUT );
		requestEditorTabs.addTab( "SOAP Request", requestEditor );
		requestEditorTabs.addTab( "Request Attachments (" + (requestAttachmentsSize == -1 ? "-" : requestAttachmentsSize) + ")", 
					new MockAttachmentPanel( mockResponse, false ));
		requestEditorTabs.addTab( "HTTP Headers (" + (requestHeadersSize == -1 ? "-" : requestHeadersSize ) + ")", 
					createRequestHttpHeadersPanel() );
		
		responseEditorTabs = new JTabbedPane( JTabbedPane.BOTTOM );
		responseEditorTabs.setTabLayoutPolicy( JTabbedPane.SCROLL_TAB_LAYOUT );
		responseEditorTabs.addTab( "Mock Response", responseEditor );
		responseEditorTabs.addTab( "Response Script", scriptEditor );
		responseEditorTabs.addTab( "Response Attachments (" + 
					mockResponse.getAttachments().length + ")", new MockAttachmentPanel( mockResponse, true ));
		responseEditorTabs.addTab(  "Response HTTP Headers", createResponseHttpHeadersPanel() );
		responseEditorTabs.setTitleAt( 3, "Response HTTP Headers (" + responseHeadersModel.getRowCount() + ")");
		
		requestTabs = new JTabbedPane();
	   requestTabPanel = UISupport.createTabPanel( requestTabs, true );
	   
	   requestEditorTabPanel = UISupport.createTabPanel(requestEditorTabs, false);
		responseEditorTabPanel = UISupport.createTabPanel(responseEditorTabs, false);
		
		JComponent component = null;
		
		if( mockResponse.getSettings().getBoolean( UISettings.START_WITH_REQUEST_TABS ))
	   {
	   	requestTabs.addTab( "Last Request", requestEditorTabPanel );
	   	requestTabs.addTab( "Mock Response", responseEditorTabPanel );
	   	splitButton.setEnabled( false );
	   	tabsButton.setSelected( true );
	   	component = requestTabPanel;
	   	
	   	requestTabs.setSelectedIndex( 1 );
	   }
	   else
	   {
	   	requestSplitPane.setTopComponent(requestEditorTabPanel);
	   	requestSplitPane.setBottomComponent(responseEditorTabPanel);
	   	requestSplitPane.setDividerLocation(0.5);
	   	component = requestSplitPane;
	   }
		
		return component;
	}

	private Component createRequestHttpHeadersPanel()
	{
		requestHeadersModel = new StringToStringMapTableModel( new StringToStringMap(), "Header", "Value", false );
		requestHeadersModel.addTableModelListener( new TableModelListener() 
		{
			public void tableChanged( TableModelEvent arg0 )
			{
				requestEditorTabs.setTitleAt( 2, "HTTP Headers (" + requestHeadersModel.getRowCount() + ")");
			}} );
		
		JTable requestHeadersTable = new JTable( requestHeadersModel );
		return new JScrollPane( requestHeadersTable );
	}

	private Component createResponseHttpHeadersPanel()
	{
		responseHeadersModel = new StringToStringMapTableModel( mockResponse.getResponseHeaders(), 
					"Header", "Value", true );
		responseHeadersModel.addTableModelListener( new TableModelListener() 
		{
			public void tableChanged( TableModelEvent arg0 )
			{
				mockResponse.setResponseHeaders( responseHeadersModel.getData() );
				responseEditorTabs.setTitleAt( 3, "Response HTTP Headers (" + responseHeadersModel.getRowCount() + ")");
			}} );
		
		responseHeadersTable = new JTable( responseHeadersModel );
		JPanel panel = new JPanel( new BorderLayout() );

		panel.add( new JScrollPane( responseHeadersTable ), BorderLayout.CENTER );
		
		ButtonBarBuilder builder = new ButtonBarBuilder();
		builder.addFixed( new JButton( new AbstractAction( "Add" ) {

			public void actionPerformed( ActionEvent arg0 )
			{
				Object header = UISupport.prompt( "Specify name of header to add", "Add HTTP Header", "" );
				if( header != null )
				{
					responseHeadersModel.add( header.toString(), "" );
				}
			}} ));
		builder.addRelatedGap();
		builder.addFixed( new JButton( new AbstractAction("Remove" ){

			public void actionPerformed( ActionEvent arg0 )
			{
				int row = responseHeadersTable.getSelectedRow();
				if( row != -1 && UISupport.confirm( "Delete selected header?", "Remove Header" ) )
				{
					responseHeadersModel.remove( row );
				}
			}} ));
		
		
		builder.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
		panel.add( builder.getPanel(), BorderLayout.SOUTH );
		
		return panel;
	}
	
	protected JComponent buildResponseScriptEditor()
	{
		responseScriptEditor = new GroovyEditor( new MockResponseGroovyEditorModel() );
		
		JPanel panel = new JPanel( new BorderLayout() );
		panel.add( buildScriptToolbar(), BorderLayout.NORTH );
		panel.add( new JScrollPane( responseScriptEditor ), BorderLayout.CENTER );
		return panel;
	}
	
	private JComponent buildScriptToolbar()
	{
		JXToolBar toolBar = UISupport.createToolbar();
		JButton runButton = UISupport.createToolbarButton( runScriptAction );
		toolBar.add( runButton );
		toolBar.add( Box.createHorizontalGlue() );
		JLabel label = new JLabel("<html>Script is invoked with <code>log</code>, <code>context</code>, <code>mockRequest</code>" +
				" and <code>mockResponse</code> variables</html>");
		label.setToolTipText( label.getText() );
		label.setMaximumSize( label.getPreferredSize() );
		
		toolBar.add( label);
		toolBar.addUnrelatedGap();
		toolBar.addFixed( createActionButton( new ShowOnlineHelpAction( HelpUrls.MOCKRESPONSE_SCRIPT_HELP_URL ), true ));
		
		return toolBar;
	}

	protected SoapMessageXmlEditor buildResponseEditor()
	{
		return new ResponseMessageEditor( new MockResponseXmlDocument( mockResponse ));
	}

	protected SoapMessageXmlEditor buildRequestEditor()
	{
		return new RequestMessageEditor( new MockRequestXmlDocument( mockResponse ));
	}

	protected JComponent buildToolbar()
	{
		JXToolBar toolbar =  UISupport.createToolbar(); 
		
		toolbar.add( openRequestButton);
		toolbar.addUnrelatedGap();
		toolbar.add(recreateButton);
		toolbar.add(createEmptyButton);
		toolbar.add(createFaultButton);
		
		createToolbar( toolbar );
		
		toolbar.add( Box.createHorizontalGlue() );
		toolbar.add( tabsButton );
		toolbar.add(splitButton);
		toolbar.add(UISupport.createToolbarButton(new ShowOnlineHelpAction(getHelpUrl())));
		
		return toolbar;
	}

	protected void createToolbar( JXToolBar toolbar )
	{
	}

	protected String getHelpUrl()
	{
		return HelpUrls.REQUESTEDITOR_HELP_URL;
	}

	protected void insertButtons(JToolBar toolbar)
	{
	}

	public void setEnabled(boolean enabled)
	{
		requestEditor.getSourceEditor().setEditable(enabled);
		responseEditor.getSourceEditor().setEditable(enabled);
		recreateButton.setEnabled(enabled);
		createEmptyButton.setEnabled(enabled);
		
		
		statusBar.setIndeterminate( !enabled );
	}

	private final class InternalPropertyChangeListener implements PropertyChangeListener
	{
		public void propertyChange(PropertyChangeEvent evt)
		{
			if( evt.getPropertyName().equals( WsdlMockResponse.MOCKRESULT_PROPERTY ))
			{
				WsdlMockRequest mockRequest = mockResponse.getMockResult().getMockRequest();
				requestEditor.getXmlDocument().setXml( mockRequest == null ? "" : mockRequest.getRequestContent());
				
				// update headers
				requestHeadersModel.setData( mockRequest.getRequestHeaders() );
				requestEditorTabs.setTitleAt( 2, "HTTP Headers (" + (mockRequest == null ? "-" : mockRequest.getRequestHeaders().size() + ")") );
				
				// update attachments
				requestEditorTabs.setTitleAt( 1, "Request Attachments (" + (mockRequest == null ? 0 :
					mockRequest.getRequestAttachments().length) + ")" );
				
				wsiValidateAction.setEnabled( true );
			}
		}
	}

	public class RequestMessageEditor extends RequestMessageXmlEditor<WsdlMockResponse>
	{
		public RequestMessageEditor(XmlDocument document)
		{
			super(document, mockResponse);
		}

		protected XmlSourceEditorView buildSourceEditor()
		{
			XmlSourceEditorView editor = getSourceEditor();
			JXEditTextArea inputArea = editor.getInputArea();

			inputArea.addFocusListener(new InputAreaFocusListener());
			
			inputArea.getInputHandler().addKeyBinding("AC+TAB", moveFocusAction);
			inputArea.getInputHandler().addKeyBinding("F5", recreateButton.getAction());
			inputArea.getInputHandler().addKeyBinding("C+F4", closePanelAction);

			return editor;
		}
	}
	
	

	public class ResponseMessageEditor extends ResponseMessageXmlEditor<WsdlMockResponse>
	{
		public ResponseMessageEditor(XmlDocument document)
		{
			super(document, mockResponse );
		}

		protected XmlSourceEditorView buildSourceEditor()
		{
			XmlSourceEditorView editor = getSourceEditor();
			
			JXEditTextArea inputArea = editor.getInputArea();
			inputArea.addFocusListener(new ResultAreaFocusListener());
			
			inputArea.getInputHandler().addKeyBinding("AC+TAB", moveFocusAction);
			inputArea.getInputHandler().addKeyBinding("C+F4", closePanelAction);
			
			JPopupMenu inputPopup = editor.getEditorPopup();
			inputPopup.insert( new JSeparator(), 2 );
			inputPopup.insert( wsiValidateAction, 3 );
			
			return editor;
		}
	}
	
	protected final class InputAreaFocusListener implements FocusListener
	{
		public void focusGained(FocusEvent e)
		{
			responseHasFocus = false;
			
			statusBar.setTarget( requestEditor.getSourceEditor().getInputArea() );
			if( !splitButton.isEnabled() )
			{
				requestTabs.setSelectedIndex( 0 );
				return;
			}
			
			if (getModelItem().getSettings().getBoolean(UISettings.NO_RESIZE_REQUEST_EDITOR))
				return;

			// dont resize if split has been dragged
			if (((SoapUISplitPaneUI) requestSplitPane.getUI()).hasBeenDragged())
				return;

			int pos = requestSplitPane.getDividerLocation();
			if (pos >= 600)
				return;
			if (requestSplitPane.getMaximumDividerLocation() > 700)
				requestSplitPane.setDividerLocation(600);
			else
				requestSplitPane.setDividerLocation(0.8);
		}

		public void focusLost(FocusEvent e)
		{
		}
	}

	protected final class ResultAreaFocusListener implements FocusListener
	{
		public void focusGained(FocusEvent e)
		{
			responseHasFocus = true;
			
			statusBar.setTarget( responseEditor.getSourceEditor().getInputArea() );
			if( !splitButton.isEnabled() )
			{
				requestTabs.setSelectedIndex( 1 );
				return;
			}
			
			if (getModelItem().getSettings().getBoolean(UISettings.NO_RESIZE_REQUEST_EDITOR))
				return;

			// dont resize if split has been dragged or result is empty
			if (((SoapUISplitPaneUI) requestSplitPane.getUI()).hasBeenDragged() )
				return;

			int pos = requestSplitPane.getDividerLocation();
			int maximumDividerLocation = requestSplitPane.getMaximumDividerLocation();
			if (pos + 600 < maximumDividerLocation)
				return;

			if (maximumDividerLocation > 700)
				requestSplitPane.setDividerLocation(maximumDividerLocation - 600);
			else
				requestSplitPane.setDividerLocation(0.2);
		}

		public void focusLost(FocusEvent e)
		{
		}
	}

	private class ClosePanelAction extends AbstractAction
	{
		public void actionPerformed(ActionEvent e)
		{
			SoapUI.getDesktop().closeDesktopPanel(getModelItem());
		}
	}

	private class MoveFocusAction extends AbstractAction
	{
		public void actionPerformed(ActionEvent e)
		{
			if (requestEditor.hasFocus())
			{
				responseEditor.requestFocus();
			}
			else
			{
				requestEditor.requestFocus();
			}
		}
	}

	public boolean dependsOn(ModelItem modelItem)
	{
		return modelItem == getModelItem() || modelItem == mockResponse.getMockOperation()
				|| modelItem == mockResponse.getMockOperation().getMockService()
				|| modelItem == mockResponse.getMockOperation().getMockService().getProject();
	}

	public String getDescription()
	{
		return "MockResponse: [" + getModelItem().getName() + "] for " + mockResponse.getMockOperation().getName();
	}
	
	private final class ChangeToTabsAction extends AbstractAction
	{
		public ChangeToTabsAction()
		{
			putValue(Action.SMALL_ICON, UISupport.createImageIcon("/toggle_tabs.gif"));
			putValue(Action.SHORT_DESCRIPTION, "Toggles to tab-based layout");
		}
		
		public void actionPerformed(ActionEvent e)
		{
			if( splitButton.isEnabled() )
			{
				splitButton.setEnabled( false );
				removeContent( requestSplitPane );
				setContent( requestTabPanel );
				requestTabs.addTab( "Last Request", requestEditorTabPanel );
				requestTabs.addTab( "Mock Response", responseEditorTabPanel );
				
				if( responseHasFocus )
				{
					requestTabs.setSelectedIndex( 1 );
					requestEditorTabs.requestFocus();
				}
				else
				{
					responseEditorTabs.requestFocus();
				}
			}
			else
			{
				int selectedIndex = requestTabs.getSelectedIndex();
				
				splitButton.setEnabled( true );
				removeContent( requestTabPanel );
				setContent( requestSplitPane );
				requestSplitPane.setTopComponent(requestEditorTabPanel);
		   	requestSplitPane.setBottomComponent(responseEditorTabPanel);
		   	requestSplitPane.setDividerLocation(0.5);
		   	
		   	if( selectedIndex == 0 )
		   		requestEditor.requestFocus();
		   	else
		   		responseEditor.requestFocus();
			}
			
			revalidate();
		}
	}

	public void setContent(JComponent content)
	{
		add( content, BorderLayout.CENTER );
	}

	public void removeContent(JComponent content)
	{
		remove( content );
	}
	
	public boolean onClose( boolean canCancel )
	{
		return release();
	}
	
	private class RunScriptAction extends AbstractAction
	{
		public RunScriptAction()
		{
			putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/run_groovy_script.gif" ));
			putValue( Action.SHORT_DESCRIPTION, "Runs this script using mock httpRequest/httpResponse objects" );
		}

		public void actionPerformed(ActionEvent e)
		{
			try
			{
				mockResponse.evaluateScript( mockResponse.getMockResult() == null ? null :
					mockResponse.getMockResult().getMockRequest() );
				
				StringToStringMap values = null;
				if( mockResponse.getMockResult() != null ) 
					values = mockResponse.getMockResult().getMockRequest().getContext().toStringToStringMap();
					
				if( values == null || values.isEmpty() )
				{
					UISupport.showInfoMessage( "No values were returned" );
				}
				else
				{
					String msg = "<html><body>Returned values:<br>";
					
					for( String name : values.keySet() )
					{
						msg += name + " : " + values.get( name ) + "<br>";
					}
					
					msg +="</body></html>";
					
					UISupport.showInfoMessage( msg );
				}
			}
			catch( RuntimeException e1 )
			{
				responseScriptEditor.selectError( e1.getMessage() );
				UISupport.showErrorMessage( e1 );
			}
		}
	}
	
	private class MockResponseGroovyEditorModel implements GroovyEditorModel
	{
		public String[] getKeywords()
		{
			return new String[] {"context", "mockRequest", "mockResponse" };
		}

		public Action getRunAction()
		{
			return runScriptAction;
		}

		public String getScript()
		{
			return mockResponse.getScript();
		}

		public void setScript( String text )
		{
			mockResponse.setScript( text );
		}

		public Settings getSettings()
		{
			return SoapUI.getSettings();
		}}
}