/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.mockoperation.actions;

import java.awt.event.ActionEvent;

import javax.swing.Action;

import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.actions.iface.AbstractSwingAction;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockOperation;
import com.eviware.soapui.model.support.ModelSupport;
import com.eviware.soapui.settings.WsdlSettings;
import com.eviware.soapui.support.UISupport;

public class OpenRequestForMockOperationAction extends AbstractSwingAction<WsdlMockOperation>
{
	public OpenRequestForMockOperationAction( WsdlMockOperation mockOperation )
	{
		super( "Open Request", "Opens/Creates a request for this MockOperation with correct endpoint", mockOperation );
		putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/open_request.gif" ) );
	}

	public void actionPerformed( ActionEvent arg0, WsdlMockOperation mockOperation)
	{
		WsdlOperation operation = mockOperation.getOperation();
		if( operation == null )
		{
			UISupport.showErrorMessage( "Missing operation for this mock response" );
			return;
		}
		
		String[] names = ModelSupport.getNames( operation.getRequests(), new String[] { "-> Create New" } );
		
		String name = ( String ) UISupport.prompt(  "Select Request for Operation [" + operation.getName() + "] " +
				"to open or create", "Open Request", names );
		if( name != null )
		{
			WsdlRequest request = operation.getRequestByName( name );
			if( request == null )
			{
				name = UISupport.prompt( "Specify name of new request", "Open Request", "Request " + (operation.getRequestCount()+1 ));
				if( name == null )
					return;
				
			   boolean createOptional = operation.getSettings().getBoolean(WsdlSettings.XML_GENERATION_ALWAYS_INCLUDE_OPTIONAL_ELEMENTS);
		      if (!createOptional)
		    	  createOptional =  UISupport.confirm(	"Create optional elements from schema?", "Create Request" );
		      
		      request = operation.addNewRequest( name );
		      String requestContent = operation.createRequest( createOptional );
		      if(requestContent != null)
		      {
		      	request.setRequestContent( requestContent );
		      }
			}
			
			request.setEndpoint( mockOperation.getMockService().getLocalEndpoint() );
			UISupport.selectAndShow( request );
		}
	}
}
