/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.submit.filters;


import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.submit.RequestFilter;
import com.eviware.soapui.impl.wsdl.submit.transports.http.BaseHttpRequestTransport;
import com.eviware.soapui.impl.wsdl.submit.transports.http.TimeablePostMethod;
import com.eviware.soapui.impl.wsdl.support.http.HttpClientSupport;
import com.eviware.soapui.model.iface.SubmitContext;
import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.settings.HttpSettings;

/**
 * RequestFilter that applies SoapUI HTTP-settings to the current request
 * 
 * @author Ole.Matzura
 */

public class HttpSettingsRequestFilter implements RequestFilter
{
	public void filterRequest(SubmitContext context, WsdlRequest wsdlRequest)
	{
		TimeablePostMethod postMethod = (TimeablePostMethod) context.getProperty( BaseHttpRequestTransport.POST_METHOD );
		
		//	 set maxsize
		Settings settings = wsdlRequest.getSettings();

		// close connections?
		if (settings.getBoolean(HttpSettings.CLOSE_CONNECTIONS))
			postMethod.setRequestHeader("Connection", "close");
		
		// max size..
		postMethod.setMaxSize( settings.getLong( HttpSettings.MAX_RESPONSE_SIZE, 0 ));
		
		// apply global settings
		HttpClientSupport.applyHttpSettings(postMethod, settings);
	}
}
