/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.testcase;

import java.lang.ref.SoftReference;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractListModel;

import com.eviware.soapui.model.testsuite.TestStepResult;

public class TestCaseLogModel extends AbstractListModel
{
	private List<Object> items = new ArrayList<Object>();
	private List<SoftReference<TestStepResult>> results = new ArrayList<SoftReference<TestStepResult>>();
	private int stepCount;

	public synchronized void addText(String msg)
	{
		items.add(msg);
		results.add(null);
		fireIntervalAdded(this, items.size() - 1, items.size()-1 );
	}

	public synchronized void addTestStepResult(TestStepResult result)
	{
		stepCount++;

		items.add("Step " + stepCount + " [" + result.getTestStep().getName() + "] " + result.getStatus() + ": took " + result.getTimeTaken() + " ms");
		SoftReference<TestStepResult> ref = new SoftReference<TestStepResult>(result);
		results.add( ref);
		for (String msg : result.getMessages())
		{
			items.add("-> " + msg);
			results.add(ref);
		}

		fireIntervalAdded(this, items.size() - 1, items.size()-1 );
	}

	public synchronized void clear()
	{
		int sz = getSize();
		items.clear();
		results.clear();
		stepCount = 0;
		fireIntervalRemoved(this, 0, sz);
	}

	public int getSize()
	{
		return items.size();
	}

	public Object getElementAt(int arg0)
	{
		try
		{
			return items.get(arg0);
		}
		catch (Throwable e)
		{
			return null;
		}			
	}

	public TestStepResult getResultAt(int index)
	{
		if( index >= results.size())
			return null;
		
		SoftReference<TestStepResult> result = results.get(index);
		return result == null ? null : result.get();
	}
}