 /*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */
 
package com.eviware.soapui.impl.wsdl.teststeps.actions;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.lang.ref.SoftReference;
import java.util.Date;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;

import com.eviware.soapui.impl.wsdl.panels.request.StringToStringMapTableModel;
import com.eviware.soapui.model.iface.MessageExchange;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.types.StringToStringMap;
import com.eviware.soapui.support.xml.JXEditTextArea;
import com.eviware.soapui.support.xml.XmlUtils;
import com.eviware.soapui.ui.desktop.DesktopPanel;
import com.eviware.soapui.ui.support.DefaultDesktopPanel;

/**
 * Shows a desktop-panel with the TestStepResult for a WsdlTestRequestStepResult
 * 
 * @author Ole.Matzura
 */

public class ShowMessageExchangeAction extends AbstractAction
{
	private DefaultDesktopPanel desktopPanel;
	private final SoftReference<MessageExchange> reference;
	private final String ownerName;

	public ShowMessageExchangeAction(MessageExchange messageExchange, String ownerName)
	{
		super( "Show Results" );
		this.ownerName = ownerName;
		this.reference = new SoftReference<MessageExchange>( messageExchange );
	}

	public void actionPerformed(ActionEvent e)
	{
		try
		{
			if( reference.get() == null )
				UISupport.showInfoMessage( "Message has been discarded.." );
			else
				UISupport.showDesktopPanel(buildFrame());
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}		
	}

	private DesktopPanel buildFrame()
	{
		if( desktopPanel == null )
		{
			desktopPanel = new DefaultDesktopPanel( "Message Viewer", 
						"Message for " + ownerName, buildContent() );
		}
		
		return desktopPanel;
	}

	private JComponent buildContent()
	{
		JTabbedPane messageTabs = new JTabbedPane();
		messageTabs.addTab( "Request Message", buildRequestTab() );
		messageTabs.addTab( "Response Message", buildResponseTab() );
		messageTabs.addTab( "Properties", buildPropertiesTab() );
		messageTabs.setPreferredSize( new Dimension( 500, 400 ));
		
		return UISupport.createTabPanel( messageTabs, true );
	}

	private Component buildPropertiesTab()
	{
		MessageExchange messageExchange = reference.get();
		StringToStringMap properties = new StringToStringMap();
		if( messageExchange != null && messageExchange.getProperties() != null )
		{
			properties.putAll( messageExchange.getProperties() );
		
			properties.put( "Timestamp", new Date( messageExchange.getTimestamp() ).toString() );
			properties.put( "Time Taken", String.valueOf( messageExchange.getTimeTaken() ) );
		}
		JTable table = new JTable( new StringToStringMapTableModel( properties, "Name", "Value", false ));
		return new JScrollPane( table );
	}

	private Component buildResponseTab()
	{
		JXEditTextArea resultArea = JXEditTextArea.createXmlEditor();
      MessageExchange messageExchange = reference.get();
      if( messageExchange != null )
      	resultArea.setText( XmlUtils.prettyPrintXml( messageExchange.getResponseContent() ) );
      else
      	resultArea.setText( "- null -" );
      resultArea.setEditable( false );
      resultArea.setToolTipText( "Response Content" );
		JScrollPane scrollPane = new JScrollPane( resultArea );
		
		if( messageExchange != null )
		{
			JSplitPane split = UISupport.createVerticalSplit( new JScrollPane( new JTable( 
						new StringToStringMapTableModel( messageExchange.getResponseHeaders(), "Header", "Value", false)) ), scrollPane );
			split.setDividerLocation( 150 );
			return split;
		}
		
		return scrollPane;
	}

	private Component buildRequestTab()
	{
		JXEditTextArea resultArea = JXEditTextArea.createXmlEditor();
      MessageExchange messageExchange = reference.get();
      if( messageExchange != null )
      	resultArea.setText( XmlUtils.prettyPrintXml( messageExchange.getRequestContent() ) );
      else 
      	resultArea.setText( "- null -" );
      resultArea.setEditable( false );
      resultArea.setToolTipText( "Request Content" );
		JScrollPane scrollPane = new JScrollPane( resultArea );
		
		if( messageExchange != null )
		{
			JSplitPane split = UISupport.createVerticalSplit( new JScrollPane( new JTable( 
						new StringToStringMapTableModel( messageExchange.getRequestHeaders(), "Header", "Value", false)) ), scrollPane );
			split.setDividerLocation( 150 );
			return split;
		}
		
		return scrollPane;
	}
}
