/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps.actions;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.util.Arrays;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestStepResult;
import com.eviware.soapui.impl.wsdl.teststeps.TransferResponseValuesTestStep.ValueTransferResult;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.ui.desktop.DesktopPanel;
import com.eviware.soapui.ui.support.DefaultDesktopPanel;

/**
 * Shows a desktop-panel with the TestStepResult for a ValueTransferResult
 * 
 * @author Ole.Matzura
 */

public class ShowTransferValuesResultsAction extends AbstractAction
{
	private final ValueTransferResult result;
	private DefaultDesktopPanel desktopPanel;

	public ShowTransferValuesResultsAction(WsdlTestStepResult result)
	{
		this.result = (ValueTransferResult) result;
	}

	public void actionPerformed(ActionEvent e)
	{
		try
		{
			if( result.isDiscarded() )
				UISupport.showInfoMessage( "Request has been discarded.." );
			else
				UISupport.showDesktopPanel(buildFrame());
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}		
	}
	
	private DesktopPanel buildFrame()
	{
		if( desktopPanel == null )
		{
			desktopPanel = new DefaultDesktopPanel( "TestStep Result", 
						"TestStep result for " + result.getTestStep().getName(), buildContent() );
		}
		
		return desktopPanel;
	}

	private JComponent buildContent()
	{
		JPanel panel = new JPanel( new BorderLayout() );
		JTable table = new JTable( new TransfersTableModel() );
		
		table.getColumnModel().getColumn( 0 ).setPreferredWidth( 150 );
		table.getColumnModel().getColumn( 1 ).setPreferredWidth( 400 );
		
		panel.add( new JScrollPane( table), BorderLayout.CENTER );
		panel.setPreferredSize( new Dimension( 550, 300 ));
		
		return panel;
	}

	private class TransfersTableModel extends AbstractTableModel
	{
		public int getRowCount()
		{
			return result.getTransferCount();
		}

		public int getColumnCount()
		{
			return 2;
		}
		
		public String getColumnName(int column)
		{
			switch( column )
			{
			case 0 : return "Transfer Name";
			case 1 : return "Transferred Values";
			}
			
			return null;
		}

		public Object getValueAt(int rowIndex, int columnIndex)
		{
			switch( columnIndex )
			{
			case 0 : return result.getTransferAt( rowIndex ).getName();
			case 1 : return Arrays.toString( result.getTransferredValuesAt( rowIndex ) );
			}
			
			return null;
		}
		
	}
	
}
