/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps.assertions;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlAnySimpleType;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;
import org.custommonkey.xmlunit.XMLAssert;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.eviware.soapui.config.RequestAssertionConfig;
import com.eviware.soapui.impl.wsdl.WsdlSubmitContext;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.panels.support.assertions.Assertable;
import com.eviware.soapui.impl.wsdl.submit.WsdlMessageExchange;
import com.eviware.soapui.impl.wsdl.submit.filters.PropertyExpansionRequestFilter;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestRunContext;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlMessageAssertion;
import com.eviware.soapui.model.iface.SubmitContext;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.ActionList;
import com.eviware.soapui.support.action.ActionSupport;
import com.eviware.soapui.support.action.DefaultActionList;
import com.eviware.soapui.support.components.JUndoableTextArea;
import com.eviware.soapui.support.xml.XmlObjectConfigurationBuilder;
import com.eviware.soapui.support.xml.XmlObjectConfigurationReader;
import com.eviware.soapui.support.xml.XmlUtils;
import com.jgoodies.forms.builder.ButtonBarBuilder;

/**
 * Assertion that matches a specified XPath expression and its expected result against
 * the associated WsdlTestRequests response message
 * 
 * @author Ole.Matzura
 */

public class XPathContainsAssertion extends WsdlMessageAssertion implements RequestAssertion, ResponseAssertion
{
	private final static Logger log = Logger.getLogger( XPathContainsAssertion.class );
	private String expectedContent;
   private String path;
	private JDialog configurationDialog;
	private JTextArea pathArea;
	private JTextArea contentArea;
	public boolean configureResult;
	public static final String ID = "XPath Match";
	public static final String LABEL = "XPath Match";

   public XPathContainsAssertion(RequestAssertionConfig assertionConfig, Assertable assertable)
   {
      super(assertionConfig, assertable);
      
      XmlObjectConfigurationReader reader = new XmlObjectConfigurationReader( getConfiguration());
      path = reader.readString( "path", null );
      expectedContent = reader.readString( "content", null );
   }
   
   public String getExpectedContent()
	{
		return expectedContent;
	}

	public void setContent(String content)
	{
		this.expectedContent = content;
	}

	public String getPath()
	{
		return path;
	}

	public void setPath(String path)
	{
		this.path = path;
	}

	protected String internalAssertResponse( WsdlMessageExchange messageExchange, SubmitContext context ) throws AssertionException
   {
		return assertContent( messageExchange.getResponseContent(), context, "Response" );
   }
	
	public String assertContent( String response, SubmitContext context, String type ) throws AssertionException
	{
      try
		{
      	if( path == null ) return "Missing path for XPath assertion";
      	if( expectedContent == null ) return "Missing content for XPath assertion";
      	
      	XmlObject xml = XmlObject.Factory.parse( response );
      	String expandedPath = PropertyExpansionRequestFilter.expandProperties( context, path );
			XmlObject[] items = xml.selectPath( expandedPath);
      	
      	XmlObject contentObj = null;
      	
      	try
			{
				contentObj = XmlObject.Factory.parse(expectedContent);
			}
			catch (Exception e)
			{
				// this is ok.. it just means that the content to match is not xml but
				// (hopefully) just a string
			}
      	
      	if( items.length == 0 )
      		throw new Exception( "Missing content for xpath [" + path + "] in " + type );
      	
      	XmlOptions options = new XmlOptions();
   		options.setSavePrettyPrint();
   		options.setSaveOuter();
   		
      	for( int c = 0; c < items.length; c++ )
      	{
      		try
				{
					if (contentObj == null)
					{
						String expandedContent = PropertyExpansionRequestFilter.expandProperties( context, expectedContent );
						if( items[c] instanceof XmlAnySimpleType )
						{
							String value = ((XmlAnySimpleType)items[c]).getStringValue();
							String expandedValue = PropertyExpansionRequestFilter.expandProperties( context, value );
							XMLAssert.assertEquals(expandedContent,expandedValue);
						}
						else
						{
							Node domNode = items[c].getDomNode();
							if (domNode.getNodeType() == Node.ELEMENT_NODE)
							{
								String expandedValue = PropertyExpansionRequestFilter.expandProperties( context, 
										XmlUtils.getElementText((Element) domNode) );
								XMLAssert.assertEquals(expandedContent, expandedValue);
							}
							else
							{
								String expandedValue = PropertyExpansionRequestFilter.expandProperties( context, 
										domNode.getNodeValue() );
								XMLAssert.assertEquals(expandedContent, expandedValue );
							}
						}
					}
					else
					{
						XMLAssert.assertXMLEqual(contentObj.xmlText(options), items[c].xmlText(options));
					}
					
					break;
				}
				catch (Throwable e)
				{
					if( c == items.length-1 )
						throw e;
				}
      	}
		}
		catch (Throwable e)
		{
			String msg = "XPathContains assertion failed for path [" + path + 
					"] : " + e.getClass().getSimpleName() + ":" + e.getMessage();
			
			throw new AssertionException( new AssertionError(msg) );
		}
      
      return type + " matches content for [" + path + "]";
   }

   public boolean configure()
   {
   	if( configurationDialog == null )
   		buildConfigurationDialog();
   	
   	pathArea.setText( path );
   	contentArea.setText( expectedContent );
   	
   	UISupport.showDialog( configurationDialog );
   	return configureResult;
   }

   protected void buildConfigurationDialog()
	{
      configurationDialog = new JDialog( UISupport.getMainFrame() );
		configurationDialog.setTitle("XPath Match configuration" );
      
      JPanel contentPanel = new JPanel( new BorderLayout() );
      contentPanel.add( UISupport.buildDescription( "Specify xpath expression and matching content", 
      			"declare namespaces with <code>declare namespace &lt;prefix&gt;='&lt;namespace&gt;';</code>", null ), BorderLayout.NORTH );
      
      JSplitPane splitPane = UISupport.createVerticalSplit();
      pathArea = new JUndoableTextArea();
      pathArea.setToolTipText( "Specifies the XPath expression to select from the message for validation" );
		splitPane.setTopComponent( UISupport.addTitledBorder( new JScrollPane( pathArea ), "XPath Expression" ));
      
      contentArea = new JUndoableTextArea();
      contentArea.setToolTipText( "Specifies the expected result of the XPath expression" );
		splitPane.setBottomComponent( UISupport.addTitledBorder(new JScrollPane( contentArea ), "Expected Result" ));
		splitPane.setDividerLocation( 100 );
      splitPane.setBorder(BorderFactory.createEmptyBorder( 0, 1, 0, 1 ));
		
      contentPanel.add( splitPane, BorderLayout.CENTER );
      
      ButtonBarBuilder builder = new ButtonBarBuilder();
      
		builder.addFixed( UISupport.createToolbarButton( new ShowOnlineHelpAction( HelpUrls.XPATHASSERTIONEDITOR_HELP_URL )));
      builder.addGlue();

      ActionList actionList = getEditorActions();
      ActionSupport.addActions( actionList, builder );
      builder.setBorder( BorderFactory.createEmptyBorder( 1, 5, 5, 5 ));
      
      contentPanel.add( builder.getPanel(), BorderLayout.SOUTH );
      
      configurationDialog.setContentPane( contentPanel );
      configurationDialog.setSize(500, 400);
      
      configurationDialog.setModal( true );
	}

	protected ActionList getEditorActions()
	{
		DefaultActionList actionList = new DefaultActionList();
		actionList.addAction( new SelectFromCurrentAction() );
		actionList.addAction( new TestPathAction() );
		actionList.addAction( new DeclareNamespacesFromCurrentAction() );
		actionList.addAction( new OkAction() );
		actionList.addAction( new CancelAction() );
		return actionList;
	}

	public XmlObject createConfiguration()
   {
		XmlObjectConfigurationBuilder builder = new XmlObjectConfigurationBuilder();
		builder.add( "path", path );
		builder.add( "content", expectedContent );
      return builder.finish();
   }

   public boolean isConfigurable()
   {
      return true;
   }
   
   public void selectFromCurrent()
	{
		try
		{
			XmlOptions options = new XmlOptions();
			options.setSavePrettyPrint();
	   	options.setSaveOuter();
	   	options.setSaveAggressiveNamespaces();
			
			XmlObject xml = XmlObject.Factory.parse(getAssertable().getAssertableContent());
			
			String txt = pathArea == null || !pathArea.isVisible() ? getPath() : pathArea.getSelectedText();
			if( txt == null ) txt = pathArea == null ? "" : pathArea.getText();
			
			WsdlTestRunContext context = new WsdlTestRunContext( getAssertable().getTestStep() );
			
			XmlObject[] items = xml.selectPath( PropertyExpansionRequestFilter.expandProperties( context, txt.trim())); 
			
			if( contentArea != null && contentArea.isVisible() )
				contentArea.setText("");
			
			if( items.length == 0 )
			{
				UISupport.showErrorMessage( "No match in current response" );
			}
			else if( items.length > 1 )
			{
				UISupport.showErrorMessage( "More than one match in current response" );
			}
			else
			{
				String stringValue = null;
				
				if( items[0] instanceof XmlAnySimpleType )
				{
					stringValue = ((XmlAnySimpleType)items[0]).getStringValue();
				}
				else
				{
					Node domNode = items[0].getDomNode();
					if (domNode.getNodeType() == Node.ELEMENT_NODE)
					{
						stringValue = items[0].xmlText( options );
					}
					else
					{
						stringValue = domNode.getNodeValue();
					}
				}
				
				if( contentArea != null && contentArea.isVisible() )
					contentArea.setText( stringValue );
				else
					setContent(stringValue );
			}
		}
		catch (Throwable e)
		{
			UISupport.showErrorMessage( e.toString() );
			e.printStackTrace();
		}
	}

	public class OkAction extends AbstractAction
	{
		public OkAction()
		{
			super( "Save" );
		}

		public void actionPerformed(ActionEvent arg0)
		{
			setPath( pathArea.getText().trim() );
			setContent( contentArea.getText() );
			setConfiguration( createConfiguration() );
			configureResult = true;
			configurationDialog.setVisible( false );
		}
	}

   public class CancelAction extends AbstractAction
   {
   	public CancelAction()
   	{
   		super( "Cancel" );
   	}
   	
		public void actionPerformed(ActionEvent arg0)
		{
			configureResult = false;
			configurationDialog.setVisible( false );
		}
   }
   
   public class DeclareNamespacesFromCurrentAction extends AbstractAction
   {
   	public DeclareNamespacesFromCurrentAction()
   	{
   		super( "Declare" );
   		putValue( Action.SHORT_DESCRIPTION, "Add namespace declaration from current message to XPath expression");
   	}
   	
		public void actionPerformed(ActionEvent arg0)
		{
			try
			{
				String content = getAssertable().getAssertableContent();
				if( content != null && content.trim().length() > 0 )
				{
					pathArea.setText( XmlUtils.declareXPathNamespaces( content ) + pathArea.getText() );
				}
				else if( UISupport.confirm( "Declare namespaces from schema instead?", "Missing Response" ))
				{
					pathArea.setText( XmlUtils.declareXPathNamespaces( getAssertable().getInterface() ) + pathArea.getText() );
				}
			}
			catch (Exception e)
			{
				log.error( e.getMessage() );
			}
		}
   }
   
   public class TestPathAction extends AbstractAction
	{
   	public TestPathAction()
   	{
   		super( "Test" );
   		putValue( Action.SHORT_DESCRIPTION, "Tests the XPath expression for the current message against the Expected Content field");
   	}
   	
		public void actionPerformed(ActionEvent arg0)
		{
			String oldPath = getPath();
			String oldContent = getExpectedContent();
			
			setPath( pathArea.getText().trim() );
			setContent( contentArea.getText() );
			
			try
			{
				String msg = assertContent( getAssertable().getAssertableContent(), 
							new WsdlSubmitContext( getAssertable().getTestStep() ), "Response"  );
				UISupport.showInfoMessage( msg, "Success" );
			}
			catch (AssertionException e)
			{
				UISupport.showErrorMessage( e.getMessage() );
			}

			setPath( oldPath );
			setContent( oldContent );
		}
	}
   
   public class SelectFromCurrentAction extends AbstractAction
	{
		public SelectFromCurrentAction()
      {
      	super( "Select from current" );
   		putValue( Action.SHORT_DESCRIPTION, "Selects the XPath expression from the current message into the Expected Content field");
      }
   	
		public void actionPerformed(ActionEvent arg0)
		{
			selectFromCurrent();			
		}
	}

	@Override
	protected String internalAssertRequest( WsdlMessageExchange messageExchange, SubmitContext context ) throws AssertionException
	{
		if( !messageExchange.hasRequest( true ) )
			return "Missing Request";
		else
			return assertContent( messageExchange.getRequestContent(), context, "Request" );
	}

	public JTextArea getContentArea()
	{
		return contentArea;
	}

	public JTextArea getPathArea()
	{
		return pathArea;
	}
}
