/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps.registry;

import java.util.HashMap;
import java.util.Map;

/**
 * Registry of WsdlTestStep factories
 * 
 * @author Ole.Matzura
 */

public class WsdlTestStepRegistry
{
	private static WsdlTestStepRegistry instance;
	private Map<String,WsdlTestStepFactory> factoryMap = new HashMap<String,WsdlTestStepFactory>();

	public WsdlTestStepRegistry()
	{
		addFactory( new WsdlTestRequestStepFactory() );
		addFactory( new TransferValuesStepFactory() );
		addFactory( new GotoStepFactory() );
		addFactory( new DelayStepFactory() );
		addFactory( new PropertiesStepFactory() );
		addFactory( new GroovyScriptStepFactory() );
	}
	
	public WsdlTestStepFactory getFactory( String type )
	{
		return factoryMap.get( type ); 
	}

	public void addFactory( WsdlTestStepFactory factory )
	{
		factoryMap.put( factory.getType(), factory );
	}

	public static synchronized WsdlTestStepRegistry getInstance()
	{
		if( instance == null )
			instance = new WsdlTestStepRegistry();
		
		return instance;
	}

	public WsdlTestStepFactory [] getFactories()
	{
		return factoryMap.values().toArray( new WsdlTestStepFactory[factoryMap.size()] );
	}
	
}
