/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */
 
 package com.eviware.soapui.support.components;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.xmlbeans.XmlException;

import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.types.StringList;

public class StringListFormComponent extends JPanel implements JFormComponent, ActionListener
{
	private DefaultListModel listModel;
	private JButton addButton;
	private JButton removeButton;
	private JList list;
	private JButton editButton;
	private Box buttonBox;
	private List<JButton> buttons = new ArrayList<JButton>();

	public StringListFormComponent( String tooltip )
	{
		this( tooltip, false );
	}
	
	public StringListFormComponent( String tooltip, boolean editOnly )
	{
		super( new BorderLayout() );
		
		listModel = new DefaultListModel();
		list = new JList(listModel);
		list.setToolTipText( tooltip );
		JScrollPane scrollPane = new JScrollPane( list);
		scrollPane.setPreferredSize( new Dimension( 300, 70 ) );
		add( scrollPane, BorderLayout.CENTER);
		buttonBox = new Box(BoxLayout.Y_AXIS);
		buttonBox.setBorder( BorderFactory.createEmptyBorder( 0, 5, 0, 0 ));
		
		if( !editOnly )
		{
			addButton = new JButton( "Add..");
			addButton.addActionListener( this );
			buttonBox.add( addButton );
			buttonBox.add( Box.createVerticalStrut( 5 ));
		}
		
		editButton = new JButton( "Edit..");
		editButton.addActionListener( this );
		buttons.add( editButton );
		buttonBox.add( editButton );
		
		if( !editOnly )
		{
			buttonBox.add( Box.createVerticalStrut( 5 ));
			removeButton = new JButton( "Remove.." );
			removeButton.addActionListener( this );
			buttonBox.add( removeButton );
			buttons.add( removeButton );
		}
			
		add( buttonBox, BorderLayout.EAST );
		
		list.addListSelectionListener( new ListSelectionListener() {

			public void valueChanged( ListSelectionEvent e )
			{
				setButtonState();
			}} );
		
		setButtonState();
	}
	
	public void addButton( Action action, boolean requireSelection )
	{
		buttonBox.add( Box.createVerticalStrut( 5 ));
		JButton button = new JButton( action );
		buttonBox.add( button);
		
		if( requireSelection )
		{
			buttons.add( button );
			setButtonState();
		}
	}
	
	public void setValue(String value)
	{
		listModel.clear();

		try
		{
			StringList stringList = StringList.fromXml( value );
			
			String[] files = stringList.toStringArray();
			for( String file : files )
				if( file.trim().length() > 0)
					listModel.addElement( file );
		}
		catch( XmlException e )
		{
			e.printStackTrace();
		}
	}

	public String getValue()
	{
		StringList result = new StringList( listModel.toArray() );
		return result.toXml();
	}

	public JList getList()
	{
		return list;
	}
	
	public void actionPerformed( ActionEvent arg0 )
	{
		if( arg0.getSource() == addButton )
		{
			String value = UISupport.prompt( "Specify value to add", "Add..", (String)null );
         if( value != null )
		   {
		     	listModel.addElement( value );
		   }
		}
		else
		{
			int selectedIndex = list.getSelectedIndex();
			
			if( arg0.getSource() == removeButton && selectedIndex != -1 )
			{
				Object elm = listModel.getElementAt( selectedIndex );
				if( UISupport.confirm( "Remove [" + elm.toString() + "] from list", "Remove" ))
				{
					listModel.remove( selectedIndex );
				}
			}
			else if( arg0.getSource() == editButton && selectedIndex != -1 )
			{
				String elm = ( String ) listModel.getElementAt( selectedIndex );
				String value = UISupport.prompt( "Specify value", "Edit..", elm );
				
				if( value != null )
				{
					listModel.setElementAt( value, selectedIndex );
				}
			}
		}
	}

	public void setButtonState()
	{
		boolean b = list.getSelectedIndex() != -1;
		for( JButton button : buttons )
			button.setEnabled( b );
	}

	public String [] getData()
	{
		String [] result = new String[listModel.size()];
		for( int c = 0; c < result.length; c++ )
			result[c] = ( String ) listModel.get( c );
		
		return result;
	}

	public void setData( String[] strings )
	{
		listModel.clear();
		for( String str : strings )
		{
			listModel.addElement( str );
		}
	}
}
